@@ -40,6 +40,10 @@ class Error(object):
             return False
         return self.level == other.level and self.message == other.message
 
+    def __hash__(self):
+        """Hash. Errors with the same level and message give the same hash."""
+        return hash(self.level) ^ hash(self.message)
+
 
 class BbpError(Error):
     """Special class of errors for BBP specification of Sonata."""
@@ -303,6 +307,9 @@ def _check_bio_nodes_group(group_df, group, config):
         errors.append(fatal('Group {} of {} misses biophysical fields: {}'.
                             format(group_name, group.file.filename, missing_fields)))
     _check_rotations()
+    if 'components' not in config:
+        errors.append(fatal('No "components" in config'))
+        return errors
     components = config['components']
     errors += _check_components_dir('morphologies_dir', components)
     errors += _check_components_dir('biophysical_neuron_models_dir', components)
@@ -331,12 +338,14 @@ def _check_nodes_group(group_df, group, config):
     Returns:
         list: List of errors, empty if no errors
     """
-    REQUIRED_GROUP_NAMES = ['model_type', 'model_template']
-    missing_fields = sorted(set(REQUIRED_GROUP_NAMES) - set(group_df.columns.tolist()))
-    if missing_fields:
-        return [fatal('Group {} of {} misses required fields: {}'
-                      .format(_get_group_name(group, parents=1), group.file.filename,
-                              missing_fields))]
+    if 'model_type' not in group_df.columns:
+        return [fatal('Group {} of {} misses "model_type" field'
+                      .format(_get_group_name(group, parents=1), group.file.filename))]
+    if group_df['model_type'][0] == 'virtual':
+        return []
+    if 'model_template' not in group_df.columns:
+        return [fatal('Group {} of {} misses "model_template" field'
+                      .format(_get_group_name(group, parents=1), group.file.filename))]
     elif group_df['model_type'][0] == 'biophysical':
         return _check_bio_nodes_group(group_df, group, config)
     return []
@@ -602,11 +611,9 @@ def validate(config_file, bbp_check=False):
     """
     config = Config(config_file).resolve()
     errors = _check_required_datasets(config)
-    if 'components' not in config:
-        errors.append(fatal('No "components" in config'))
     if not errors:
         errors = _check_populations(config)
     if not bbp_check:
         errors = [e for e in errors if not isinstance(e, BbpError)]
     _print_errors(errors)
-    return errors
+    return set(errors)
