@@ -25,23 +25,24 @@ import neurom as nm
 from bluepysnap.settings import MORPH_CACHE_SIZE
 from bluepysnap.sonata_constants import Node
 from bluepysnap.exceptions import BluepySnapError
+from bluepysnap.circuit_ids import CircuitNodeId
 
 
 class MorphHelper(object):
     """Collection of morphology-related methods."""
 
-    def __init__(self, morph_dir, nodes):
+    def __init__(self, morph_dir, population):
         """Initializes a MorphHelper object from a directory path and a NodePopulation object.
 
         Args:
             morph_dir (str): Path to the directory containing the node morphologies.
-            nodes (NodePopulation): NodePopulation object used to query the nodes.
+            population (NodePopulation): NodePopulation object used to query the nodes.
 
         Returns:
             MorphHelper: A MorphHelper object.
         """
         self._morph_dir = morph_dir
-        self._nodes = nodes
+        self._population = population
 
         # all nodes from a population must have the same model type
         if not self._is_biophysical(0):
@@ -55,23 +56,31 @@ class MorphHelper(object):
             self._load = lru_cache(maxsize=MORPH_CACHE_SIZE)(self._load)
 
     def _is_biophysical(self, node_id):
-        return self._nodes.get(node_id, Node.MODEL_TYPE) == "biophysical"
+        return self._population.get(node_id, Node.MODEL_TYPE) == "biophysical"
 
     def get_filepath(self, node_id):
-        """Return path to SWC morphology file corresponding to `node_id`."""
-        name = self._nodes.get(node_id, Node.MORPHOLOGY)
-        return os.path.join(self._morph_dir, "%s.swc" % name)
+        """Return path to SWC morphology file corresponding to `node_id`.
+
+        Args:
+            node_id (int/CircuitNodeId): could be a int or CircuitNodeId.
+        """
+        name = self._population.get(node_id, Node.MORPHOLOGY)
+        if isinstance(node_id, (int, CircuitNodeId)):
+            return os.path.join(self._morph_dir, "%s.swc" % name)
+        raise BluepySnapError("node_id must be a int or a CircuitNodeId")
 
     def get(self, node_id, transform=False):
         """Return NeuroM morphology object corresponding to `node_id`.
 
-        If `transform` is True, rotate and translate morphology points
-        according to `node_id` position in the circuit.
+        Args:
+            node_id (int/CircuitNodeId): could be a single int or a CircuitNodeId.
+            transform (bool): If `transform` is True, rotate and translate morphology points
+                according to `node_id` position in the circuit.
         """
         filepath = self.get_filepath(node_id)
         result = self._load(filepath)
         if transform:
-            A_t = self._nodes.orientations(node_id).transpose()
-            B = self._nodes.positions(node_id).values
+            A_t = self._population.orientations(node_id).transpose()
+            B = self._population.positions(node_id).values
             result = result.transform(lambda p: np.dot(p, A_t) + B)
         return result
