@@ -32,12 +32,213 @@ from bluepysnap import utils
 from bluepysnap.exceptions import BluepySnapError
 from bluepysnap.sonata_constants import (DYNAMICS_PREFIX, NODE_ID_KEY,
                                          POPULATION_KEY, Node, ConstContainer)
-
+from bluepysnap.circuit_ids import CircuitNodeId, CircuitNodeIds
 
 # this constant is not part of the sonata standard
 NODE_SET_KEY = "$node_set"
 
 
+class Nodes(object):
+    """The top level Nodes accessor."""
+
+    def __init__(self, circuit):
+        """Initialize the top level Nodes accessor."""
+        self._circuit = circuit
+        self._config = self._circuit.config['networks']['nodes']
+        self._populations = self._get_populations()
+
+    def _get_populations(self):
+        """Collect the different NodePopulation."""
+        res = {}
+        for file_config in self._config:
+            storage = NodeStorage(file_config, self._circuit)
+            for population in storage.population_names:  # pylint: disable=not-an-iterable
+                if population in res:
+                    raise BluepySnapError("Duplicated node population: '%s'" % population)
+                res[population] = storage.population(population)
+        return res
+
+    @cached_property
+    def population_names(self):
+        """Returns all the population names from the Circuit."""
+        return sorted(self._populations)
+
+    @cached_property
+    def property_dtypes(self):
+        """Returns all the property dtypes for the Circuit."""
+        def _update(d, index, value):
+            if index not in d:
+                d[index] = value
+            elif d[index] != value:
+                raise BluepySnapError("Same property with different "
+                                      "dtype. {}: {}!= {}".format(index, value, d[index]))
+
+        res = dict()
+        for pop in self.populations():
+            for varname, dtype in pop.property_dtypes.iteritems():
+                _update(res, varname, dtype)
+        return pd.Series(res)
+
+    def keys(self):
+        """Returns iterator on the population names.
+
+        Made to simulate the behavior of a dict.keys().
+        """
+        for name in self.population_names:
+            yield name
+
+    def values(self):
+        """Returns iterator on the NodePopulations.
+
+        Made to simulate the behavior of a dict.values().
+        """
+        for name in self.population_names:
+            yield self[name]
+
+    def items(self):
+        """Returns iterator on the tuples (population name, NodePopulations).
+
+        Made to simulate the behavior of a dict.items().
+        """
+        for name in self.population_names:
+            yield name, self[name]
+
+    # helper renaming
+    names = keys
+    populations = values
+
+    def __getitem__(self, population_name):
+        """Access the NodePopulation corresponding to the population 'population_name'."""
+        try:
+            return self._populations[population_name]
+        except KeyError:
+            raise BluepySnapError("{} not a node population.".format(population_name))
+
+    def __iter__(self):
+        """Allows iteration over the different NodePopulation."""
+        return iter(self.names())
+
+    @cached_property
+    def size(self):
+        """Total number of nodes inside the circuit."""
+        return sum(pop.size for pop in self.populations())
+
+    @cached_property
+    def property_names(self):
+        """Returns all the properties present inside the circuit."""
+        return set(prop for pop in self.populations() for prop in pop.property_names)
+
+    def property_values(self, prop):
+        """Returns all the values for a given property."""
+        return set(value for pop in self.populations() if prop in pop.property_names for value in
+                   pop.property_values(prop))
+
+    def ids(self, group=None):
+        """Returns the CircuitNodeIds corresponding to the nodes from ``group``.
+
+        Args:
+            group (CircuitNodeId/CircuitNodeIds/int/sequence/str/mapping/None): Which IDs will be
+            returned depends on the type of the ``group`` argument:
+                - ``CircuitNodeId``: return the ID in a CircuitNodeIds object if it belongs to
+                    the circuit.
+                - ``CircuitNodeIds``: return the IDs in a CircuitNodeIds object if they belong to
+                    the circuit.
+                - ``int``: if the node ID is present in all populations, returns a CircuitNodeIds
+                    object containing the corresponding node ID for all populations.
+                - ``sequence``: if all the values contained in the sequence are present in all
+                    populations, returns a CircuitNodeIds object containing the corresponding node
+                    IDs for all populations.
+                - ``str``: use a node set name as input. Returns a CircuitNodeIds object containing
+                    nodes selected by the node set.
+                - ``mapping``: Returns a CircuitNodeIds object containing nodes matching a
+                    properties filter.
+                - ``None``: return all node IDs of the circuit in a CircuitNodeIds object.
+
+        Returns:
+            CircuitNodeIds: returns a CircuitNodeIds containing all the node IDs and the
+                corresponding populations. All the explicitly requested IDs must be present inside
+                the circuit.
+
+        Raises:
+            BluepySnapError: when a population from a CircuitNodeIds is not present in the circuit.
+            BluepySnapError: when an id query via a int, sequence, or CircuitNodeIds is not present
+                in the circuit.
+
+        Examples:
+            The available group parameter values (example with 2 node populations pop1 and pop2):
+            >>> nodes = circuit.nodes
+            >>> nodes.ids(group=None)  #  returns all CircuitNodeIds from the circuit
+            >>> node_ids = CircuitNodeIds.from_arrays(["pop1", "pop2"], [1, 3])
+            >>> nodes.ids(group=node_ids)  #  returns ID 1 from pop1 and ID 3 from pop2
+            >>> nodes.ids(group=0)  #  returns CircuitNodeIds 0 from pop1 and pop2
+            >>> nodes.ids(group=[0, 1])  #  returns CircuitNodeIds 0 and 1 from pop1 and pop2
+            >>> nodes.ids(group="node_set_name")  # returns CircuitNodeIds matching node set
+            >>> nodes.ids(group={Node.LAYER: 2})  # returns CircuitNodeIds matching layer==2
+            >>> nodes.ids(group={Node.LAYER: [2, 3]})  # returns CircuitNodeIds with layer in [2,3]
+            >>> nodes.ids(group={Node.X: (0, 1)})  # returns CircuitNodeIds with 0 < x < 1
+            >>> # returns CircuitNodeIds matching one of the queries inside the 'or' list
+            >>> nodes.ids(group={'$or': [{ Node.LAYER: [2, 3]},
+            >>>                          { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
+            >>> # returns CircuitNodeIds matching all the queries inside the 'and' list
+            >>> nodes.ids(group={'$and': [{ Node.LAYER: [2, 3]},
+            >>>                           { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
+        """
+        if isinstance(group, CircuitNodeIds):
+            diff = np.setdiff1d(group.get_populations(unique=True), self.population_names)
+            if diff.size != 0:
+                raise BluepySnapError("Population {} does not exist in the circuit.".format(diff))
+
+        ids = np.empty((0,), dtype=np.int64)
+        str_type = "<U{}".format(max(len(pop) for pop in self.population_names))
+        populations = np.empty((0,), dtype=str_type)
+        for name, pop in self.items():
+            pop_ids = pop.ids(group=group, raise_missing_property=False)
+            pops = np.full_like(pop_ids, fill_value=name, dtype=str_type)
+            ids = np.concatenate([ids, pop_ids])
+            populations = np.concatenate([populations, pops])
+        return CircuitNodeIds.from_arrays(populations, ids)
+
+    def get(self, group=None, properties=None):
+        """Node properties as a pandas DataFrame.
+
+        Args:
+            group (CircuitNodeIds/int/sequence/str/mapping/None): Which nodes will have their
+                properties returned depends on the type of the ``group`` argument:
+                See :py:class:`~bluepysnap.nodes.Nodes.ids`.
+
+            properties (str/list): If specified, return only the properties in the list.
+                Otherwise return all properties.
+
+        Returns:
+            pandas.DataFrame: Return a pandas DataFrame indexed by NodeCircuitIds containing the
+                properties from ``properties``.
+
+        Notes:
+            The NodePopulation.property_names function will give you all the usable properties
+            for the `properties` argument.
+        """
+        ids = self.ids(group)
+        if properties is None:
+            properties = self.property_names
+        properties = utils.ensure_list(properties)
+
+        unknown_props = set(properties) - self.property_names
+        if unknown_props:
+            raise BluepySnapError("Unknown properties required: {}".format(unknown_props))
+
+        res = pd.DataFrame(index=ids.index, columns=properties)
+        for name, pop in self.items():
+            global_pop_ids = ids.filter_population(name)
+            pop_ids = global_pop_ids.get_ids()
+            pop_properties = set(properties) & pop.property_names
+            # indices from NodePopulation and Node get functions are different so I cannot use
+            # a dataframe equal directly and properties have different types so cannot use a multi
+            # dim numpy array
+            for prop in pop_properties:
+                res.loc[global_pop_ids.index, prop] = pop.get(pop_ids, properties=prop).to_numpy()
+        return res.sort_index()
+
+
 class NodeStorage(object):
     """Node storage access."""
 
@@ -250,7 +451,8 @@ class NodePopulation(object):
     def _check_id(self, node_id):
         """Check that single node ID belongs to the circuit."""
         if node_id not in self._data.index:
-            raise BluepySnapError("node ID not found: %d" % node_id)
+            raise BluepySnapError("node ID not found: {} in population"
+                                  " '{}'".format(node_id, self.name))
 
     def _check_ids(self, node_ids):
         """Check that node IDs belong to the circuit."""
@@ -264,7 +466,8 @@ class NodePopulation(object):
             max_id = max(node_ids)
             min_id = min(node_ids)
         if min_id < 0 or max_id >= self._data.index.shape[0]:
-            raise BluepySnapError("All node IDs must be >= 0 and < %s" % self._data.index.shape[0])
+            raise BluepySnapError("All node IDs must be >= 0 and < {} for population "
+                                  "'{}'".format(self._data.index.shape[0], self.name))
 
     def _check_property(self, prop):
         """Check if a property exists inside the dataset."""
@@ -309,13 +512,16 @@ class NodePopulation(object):
             node_ids = np.intersect1d(node_ids, self.ids(node_set))
         return queries, self._positional_mask(node_ids)
 
-    def _properties_mask(self, queries):
+    def _properties_mask(self, queries, raise_missing_prop):
         """Return mask of node IDs with rows matching `props` dict."""
         # pylint: disable=assignment-from-no-return
         circuit_keys = {POPULATION_KEY, NODE_ID_KEY, NODE_SET_KEY}
         unknown_props = set(queries) - set(self._data.columns) - circuit_keys
         if unknown_props:
-            raise BluepySnapError("Unknown node properties: [{0}]".format(", ".join(unknown_props)))
+            if raise_missing_prop:
+                raise BluepySnapError(
+                    "Unknown node properties: [{0}]".format(", ".join(unknown_props)))
+            return np.full(len(self._data), fill_value=False)
 
         queries, mask = self._circuit_mask(queries)
         if not mask.any():
@@ -336,7 +542,7 @@ class NodePopulation(object):
             mask = np.logical_and(mask, prop_mask)
         return mask
 
-    def _operator_mask(self, queries):
+    def _operator_mask(self, queries, raise_missing_prop):
         """Handle the query operators '$or', '$and'."""
         if len(queries) == 0:
             return np.full(len(self._data), True)
@@ -351,14 +557,14 @@ class NodePopulation(object):
             queries = queries.pop("$and")
             operator = np.logical_and
         else:
-            return self._properties_mask(queries)
+            return self._properties_mask(queries, raise_missing_prop)
 
         mask = np.full(len(self._data), first_key != "$or")
         for query in queries:
-            mask = operator(mask, self._operator_mask(query))
+            mask = operator(mask, self._operator_mask(query, raise_missing_prop))
         return mask
 
-    def _node_ids_by_filter(self, queries):
+    def _node_ids_by_filter(self, queries, raise_missing_prop):
         """Return node IDs if their properties match the `queries` dict.
 
         `props` values could be:
@@ -375,16 +581,17 @@ class NodePopulation(object):
             >>>                              { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
 
         """
-        return self._data.index[self._operator_mask(queries)].values
+        return self._data.index[self._operator_mask(queries, raise_missing_prop)].values
 
-    def ids(self, group=None, limit=None, sample=None):
+    def ids(self, group=None, limit=None, sample=None, raise_missing_property=True):
         """Node IDs corresponding to node ``group``.
 
         Args:
-            group (int/sequence/str/mapping/None): Which IDs will be returned
-                depends on the type of the ``group`` argument:
+            group (int/CircuitNodeId/CircuitNodeIds/sequence/str/mapping/None): Which IDs will be
+                returned depends on the type of the ``group`` argument:
 
-                - ``int``: return a single node ID if it belongs to the circuit.
+                - ``int``, ``CircuitNodeId``: return a single node ID if it belongs to the circuit.
+                - ``CircuitNodeIds`` return IDs of nodes in an array.
                 - ``sequence``: return IDs of nodes in an array.
                 - ``str``: return IDs of nodes in a node set.
                 - ``mapping``: return IDs of nodes matching a properties filter.
@@ -394,10 +601,15 @@ class NodePopulation(object):
                 Otherwise the result is sorted and contains no duplicates.
 
             sample (int): If specified, randomly choose ``sample`` number of
-                IDs from the match result.
+                IDs from the match result. If the size of the sample is greater than
+                the size of the NodePopulation then all ids are taken and shuffled.
 
             limit (int): If specified, return the first ``limit`` number of
-                IDs from the match result.
+                IDs from the match result. If limit is greater than the size of the population
+                all node IDs are returned.
+
+            raise_missing_property (bool): if True, raises if a property is not listed in this
+                population. Otherwise the ids are just not selected if a property is missing.
 
         Returns:
             numpy.array: A numpy array of IDs.
@@ -408,7 +620,12 @@ class NodePopulation(object):
             >>> nodes.ids(group=None)  #  returns all IDs
             >>> nodes.ids(group={})  #  returns all IDs
             >>> nodes.ids(group=1)  #  returns the single ID if present in population
+            >>> #  returns the single ID if present in population and the circuit id population
+            >>> #  corresponds to nodes.name
+            >>> nodes.ids(group=CircuitNodeId('pop', 1))
             >>> nodes.ids(group=[1,2,3])  # returns list of IDs if all present in population
+            >>> #  returns list of IDs if all present in population
+            >>> nodes.ids(group=CircuitNodeIds.from_dict({"pop": [0, 1,2]}))
             >>> nodes.ids(group="node_set_name")  # returns list of IDs matching node set
             >>> nodes.ids(group={ Node.LAYER: 2})  # returns list of IDs matching layer==2
             >>> nodes.ids(group={ Node.LAYER: [2, 3]})  # returns list of IDs with layer in [2,3]
@@ -420,25 +637,36 @@ class NodePopulation(object):
             >>> nodes.ids(group={'$and': [{ Node.LAYER: [2, 3]},
             >>>                           { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
         """
+        # pylint: disable=too-many-branches
         preserve_order = False
         if isinstance(group, six.string_types):
             group = self._get_node_set(group)
+        elif isinstance(group, CircuitNodeIds):
+            group = group.filter_population(self.name).get_ids()
 
         if group is None:
             result = self._data.index.values
         elif isinstance(group, collections.Mapping):
-            result = self._node_ids_by_filter(queries=group)
+            result = self._node_ids_by_filter(queries=group,
+                                              raise_missing_prop=raise_missing_property)
         elif isinstance(group, np.ndarray):
             result = group
             self._check_ids(result)
             preserve_order = True
         else:
             result = utils.ensure_list(group)
+            if isinstance(next(iter(result), None), CircuitNodeId):
+                try:
+                    result = [cid.id for cid in result if cid.population == self.name]
+                except AttributeError:
+                    raise BluepySnapError("All values from a list must be of type int or "
+                                          "CircuitNodeId.")
             self._check_ids(result)
             preserve_order = isinstance(group, collections.Sequence)
+
         if sample is not None:
             if len(result) > 0:
-                result = np.random.choice(result, sample, replace=False)
+                result = np.random.choice(result, min(sample, len(result)), replace=False)
             preserve_order = False
         if limit is not None:
             result = result[:limit]
@@ -453,22 +681,24 @@ class NodePopulation(object):
         """Node properties as a pandas Series or DataFrame.
 
         Args:
-            group (int/sequence/str/mapping/None): Which nodes will have their properties
-                returned depends on the type of the ``group`` argument:
+            group (int/CircuitNodeId/CircuitNodeIds/sequence/str/mapping/None): Which nodes will
+                have their properties returned depends on the type of the ``group`` argument:
 
-                - ``int``: return the properties of a single node.
+                - ``int``, ``CircuitNodeId``: return the properties of a single node.
+                - ``CircuitNodeIds`` return the properties from a NodeCircuitNodeIds.
                 - ``sequence``: return the properties from a list of node.
                 - ``str``: return the properties of nodes in a node set.
                 - ``mapping``: return the properties of nodes matching a properties filter.
                 - ``None``: return the properties of all nodes.
 
-            properties (set): If specified, return only the properties in the set.
+            properties (list): If specified, return only the properties in the list.
                 Otherwise return all properties.
 
         Returns:
-            pandas.Series/pandas.DataFrame:
-                If single node ID is passed as ``group`` returns a pandas Series.
-                Otherwise return a pandas DataFrame indexed by node IDs.
+            value/pandas.Series/pandas.DataFrame:
+                If single node ID is passed as ``group`` and single property as properties returns
+                a single value. If single node ID is passed as ``group`` and list as property
+                returns a pandas Series. Otherwise return a pandas DataFrame indexed by node IDs.
 
         Notes:
             The NodePopulation.property_names function will give you all the usable properties
@@ -483,6 +713,8 @@ class NodePopulation(object):
         if group is not None:
             if isinstance(group, six.integer_types + (np.integer,)):
                 self._check_id(group)
+            elif isinstance(group, CircuitNodeId):
+                group = self.ids(group)[0]
             else:
                 group = self.ids(group)
             result = result.loc[group]
@@ -493,10 +725,11 @@ class NodePopulation(object):
         """Node position(s) as pandas Series or DataFrame.
 
         Args:
-            group (int/sequence/str/mapping/None): Which nodes will have their positions
-                returned depends on the type of the ``group`` argument:
+            group (int/CircuitNodeId/CircuitNodeIds/sequence/str/mapping/None): Which nodes will
+                have their positions returned depends on the type of the ``group`` argument:
 
-                - ``int``: return the position of a single node.
+                - ``int``, ``CircuitNodeId``: return the position of a single node.
+                - ``CircuitNodeIds`` return the position from a NodeCircuitNodeIds.
                 - ``sequence``: return the positions from a list of node IDs.
                 - ``str``: return the positions of nodes in a node set.
                 - ``mapping``: return the positions of nodes matching a properties filter.
@@ -516,10 +749,11 @@ class NodePopulation(object):
         """Node orientation(s) as a pandas numpy array or pandas Series.
 
         Args:
-            group (int/sequence/str/mapping/None): Which nodes will have their positions
-                returned depends on the type of the ``group`` argument:
+            group (int/CircuitNodeId/CircuitNodeIds/sequence/str/mapping/None): Which nodes will
+                have their positions returned depends on the type of the ``group`` argument:
 
-                - ``int``: return the orientation of a single node.
+                - ``int``, ``CircuitNodeId``: return the orientation of a single node.
+                - ``CircuitNodeIds`` return the orientation from a NodeCircuitNodeIds.
                 - ``sequence``: return the orientations from a list of node IDs.
                 - ``str``: return the orientations of nodes in a node set.
                 - ``mapping``: return the orientations of nodes matching a properties filter.
@@ -563,7 +797,7 @@ class NodePopulation(object):
             return result.get(prop, np.zeros((result.shape[0],)))
 
         args = [_get_values(prop) for prop in props]
-        if isinstance(group, six.integer_types + (np.integer,)):
+        if isinstance(group, six.integer_types + (np.integer, CircuitNodeId)):
             return trans(*args)[0]
         return pd.Series(trans(*args), index=result.index, name='orientation')
 
@@ -571,10 +805,11 @@ class NodePopulation(object):
         """Total number of nodes for a given node group.
 
         Args:
-            group (int/sequence/str/mapping/None): Which nodes will have their positions
-                returned depends on the type of the ``group`` argument:
+            group (int/CircuitNodeId/CircuitNodeIds/sequence/str/mapping/None): Which nodes will
+                have their positions returned depends on the type of the ``group`` argument:
 
-                - ``int``: return the count of a single node.
+                - ``int``, ``CircuitNodeId``: return the count of a single node.
+                - ``CircuitNodeIds`` return the count of nodes from a NodeCircuitNodeIds.
                 - ``sequence``: return the count of nodes from a list of node IDs.
                 - ``str``: return the count of nodes in a node set.
                 - ``mapping``: return count of nodes matching a properties filter.
